@@ -1,28 +1,28 @@
 package Text::Template::Simple::Base::Compiler;
 use strict;
 use vars qw($VERSION);
-use Carp qw( croak );
-use Text::Template::Simple::Util;
-use Text::Template::Simple::Constants;
+use Text::Template::Simple::Util qw(:all);
+use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 sub _compiler { shift->[SAFE] ? COMPILER_SAFE : COMPILER }
 
 sub _compile {
    my $self  = shift;
-   my $tmpx  = shift || croak "No template specified";
+   my $tmpx  = shift || fatal('tts.base.compiler._compile.notmp');
    my $param = shift || [];
    my $opt   = shift || {};
 
-   croak "params must be an arrayref!" if not isaref($param);
-   croak "opts must be a hashref!"     if not ishref($opt);
+   fatal('tts.base.compiler._compile.param') if not isaref($param);
+   fatal('tts.base.compiler._compile.opt')   if not ishref($opt  );
 
    # set defaults
    $opt->{id}       ||= ''; # id is AUTO
    $opt->{map_keys} ||= 0;  # use normal behavior
    $opt->{chkmt}    ||= 0;  # check mtime of file template?
    $opt->{_sub_inc} ||= 0;  # are we called from a dynamic include op?
+   $opt->{_filter}  ||= ''; # any filters?
 
    my $tmp = $self->_examine( $tmpx );
    return $tmp if $self->[TYPE] eq 'ERROR';
@@ -30,11 +30,12 @@ sub _compile {
    if ( $opt->{_sub_inc} ) {
       # TODO:generate a single error handler for includes, merge with _include()
       # tmpx is a "file" included from an upper level compile()
-      my $etitle = $self->_include_error('dynamic');
-      my $exists = $self->_file_exists( $tmpx );
+      my $etitle = $self->_include_error( T_DYNAMIC );
+      my $exists = $self->io->file_exists( $tmpx );
       return $etitle . " '$tmpx' is not a file" if not $exists;
       # TODO: remove this second call somehow, reduce  to a single call
       $tmp = $self->_examine( $exists ); # re-examine
+      $self->[NEEDS_OBJECT]++; # interpolated includes will need that
    }
 
    if ( $opt->{chkmt} ) {
@@ -53,7 +54,7 @@ sub _compile {
    my($CODE, $ok);
    my $cache_id = '';
 
-   my $as_is = $opt->{_sub_inc} && $opt->{_sub_inc} eq 'static';
+   my $as_is = $opt->{_sub_inc} && $opt->{_sub_inc} == T_STATIC;
 
    if ( $self->[CACHE] ) {
       my $method = $opt->{id};
@@ -86,12 +87,37 @@ sub _compile {
    push @args, $self if $self->[NEEDS_OBJECT];
    push @args, @{ $self->[ADD_ARGS] } if $self->[ADD_ARGS];
    push @args, @{ $param };
-   return $CODE->( @args );
+   my $out = $CODE->( @args );
+
+   if ( $opt->{_filter} ) {
+      $self->_call_filters( \$out, split RE_FILTER_SPLIT, $opt->{_filter} );
+   }
+
+   return $out;
+}
+
+sub _call_filters {
+   my $self    = shift;
+   my $oref    = shift;
+   my @filters = @_;
+   my $fname   = $self->[FILENAME];
+   my $fbase   = 'Text::Template::Simple::Dummy';
+
+   APPLY_FILTERS: foreach my $filter ( @filters ) {
+      my $fref = $fbase->can( "filter_" . $filter );
+      if ( ! $fref ) {
+         $$oref .= "\n[ filter warning ] Can not apply undefined filter $filter to $fname\n";
+         next;
+      }
+      $fref->( $self, $oref );
+   }
+
+   return;
 }
 
 sub _wrap_compile {
    my $self   = shift;
-   my $parsed = shift or croak "nothing to compile";
+   my $parsed = shift or fatal('tts.base.compiler._wrap_compile.parsed');
    LOG( CACHE_ID => $self->cache->id ) if $self->[WARN_IDS] && $self->cache->id;
    LOG( COMPILER => $self->[SAFE] ? 'Safe' : 'Normal' ) if DEBUG();
    my($CODE, $error);
@@ -102,7 +128,7 @@ sub _wrap_compile {
       my $error2;
       if ( $self->[RESUME] ) {
          $CODE =  sub {
-                     sprintf ("[%s Fatal Error] %s", $self->_class_id, $error )
+                     sprintf("[%s Fatal Error] %s", $self->_class_id, $error )
                   };
          $error2 = $@;
       }
@@ -115,12 +141,12 @@ sub _wrap_compile {
 sub _mini_compiler {
    # little dumb compiler for internal templates
    my $self     = shift;
-   my $template = shift || croak "_mini_compiler(): missing the template";
-   my $param    = shift || croak "_mini_compiler(): missing the parameters";
+   my $template = shift || fatal('tts.base.compiler._mini_compiler.notmp');
+   my $param    = shift || fatal('tts.base.compiler._mini_compiler.noparam');
    my $opt      = shift || {};
 
-   croak "_mini_compiler(): options must be a hash"    if ref($opt)   ne 'HASH';
-   croak "_mini_compiler(): parameters must be a HASH" if ref($param) ne 'HASH';
+   fatal('tts.base.compiler._mini_compiler.opt')   if ! ishref($opt  );
+   fatal('tts.base.compiler._mini_compiler.param') if ! ishref($param);
 
    foreach my $var ( keys %{ $param } ) {
       $template =~ s[<%\Q$var\E%>][$param->{$var}]xmsg;
@@ -144,6 +170,9 @@ Private module.
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Base::Compiler>
+released on C<26 April 2009>.
+
 Private module.
 
 =head1 AUTHOR
