@@ -1,11 +1,10 @@
 package Text::Template::Simple::Base::Include;
 use strict;
 use vars qw($VERSION);
-use Carp qw( croak );
-use Text::Template::Simple::Util;
-use Text::Template::Simple::Constants;
+use Text::Template::Simple::Util qw(:all);
+use Text::Template::Simple::Constants qw(:all);
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 sub _include_no_monolith {
    # no monolith eh?
@@ -29,7 +28,7 @@ sub _include_static {
    my($self, $file, $text, $err) = @_;
    return $self->[MONOLITH]
         ? 'q~' . escape('~' => $text) . '~;'
-        : $self->_include_no_monolith( static => $file )
+        : $self->_include_no_monolith( T_STATIC, $file )
         ;
 }
 
@@ -47,16 +46,16 @@ sub _include_dynamic {
       my $max   = MAX_RECURSION;
       my $error = qq{$err Deep recursion (>=$max) detected in }
                 . qq{the included file: $file};
-      LOG( DEEP_RECURSION => $file ) if DEBUG;
-      $error = escape '~' => $error;
+      LOG( DEEP_RECURSION => $file ) if DEBUG();
+      $error = escape( '~' => $error );
       $self->[DEEP_RECURSION] = 1;
       $rv .= "q~$error~";
    }
    else {
-      # do stuff is for file name access through $0 in templates
+      # local stuff is for file name access through $0 in templates
       $rv .= $self->[MONOLITH]
            ? do { local $self->[FILENAME] = $file; $self->_parse( $text ) }
-           : $self->_include_no_monolith( dynamic => $file )
+           : $self->_include_no_monolith( T_DYNAMIC, $file )
            ;
    }
 
@@ -67,18 +66,18 @@ sub _include_dynamic {
 
 sub _include {
    my $self       = shift;
-   my $type       = shift || '';
+   my $type       = shift || 0;
    my $file       = shift;
-      $type       = lc $type;
-      $file       = trim $file;
-   my $is_static  = $type eq 'static';
-   my $is_dynamic = $type eq 'dynamic';
+   my $is_static  = T_STATIC  == $type ? 1 : 0;
+   my $is_dynamic = T_DYNAMIC == $type ? 1 : 0;
    my $known      = $is_static || $is_dynamic;
 
-   croak "Unknown include type: $type" if not $known;
+   fatal('tts.base.include._include.unknown', $type) if not $known;
+
+   $file = trim $file;
 
    my $err    = $self->_include_error( $type );
-   my $exists = $self->_file_exists( $file );
+   my $exists = $self->io->file_exists( $file );
    my $interpolate;
 
    if ( $exists ) {
@@ -89,27 +88,32 @@ sub _include {
       $interpolate = 1; # just guessing ...
    }
 
-   if ( -d $file ) {
-      $file = escape '~' => $file;
-      return "q~$err '$file' is a directory~";
+   return "q~$err '" . escape('~' => $file) . "' is a directory~"
+      if $self->io->is_dir( $file );
+
+   if ( DEBUG() ) {
+      require Text::Template::Simple::Tokenizer;
+      my $toke =  Text::Template::Simple::Tokenizer->new(
+                     @{ $self->[DELIMITERS] },
+                     $self->[PRE_CHOMP],
+                     $self->[POST_CHOMP]
+                  );
+      LOG( INCLUDE => $toke->_visualize_tid($type) . " => '$file'" );
    }
 
-   LOG( INCLUDE => "$type => '$file'" ) if DEBUG();
-
-   my $text;
    if ( $interpolate ) {
       my $rv = $self->_interpolate( $file, $type );
       $self->[NEEDS_OBJECT]++;
       LOG(INTERPOLATE_INC => "TYPE: $type; DATA: $file; RV: $rv") if DEBUG();
       return $rv;
    }
-   else {
-      eval { $text = $self->io->slurp($file) };
-      return "q~$err $@~" if $@;
-   }
 
-   return $self->_include_dynamic( $file, $text, $err) if $is_dynamic;
-   return $self->_include_static(  $file, $text, $err);
+   my $text;
+   eval { $text = $self->io->slurp($file); };
+   return "q~$err $@~" if $@;
+
+   my $meth = '_include_' . ($is_dynamic ? 'dynamic' : 'static');
+   return $self->$meth( $file, $text, $err);
 }
 
 sub _interpolate {
@@ -117,23 +121,43 @@ sub _interpolate {
    my $file   = shift;
    my $type   = shift;
    my $etitle = $self->_include_error($type);
-   my $rv     = $self->_mini_compiler(
-                  $self->_internal('sub_include') => {
-                     OBJECT      => $self->[FAKER_SELF],
-                     INCLUDE     => escape( q{'} => $file   ),
-                     ERROR_TITLE => escape( q{'} => $etitle ),
-                     TYPE        => $type,
-                  } => {
-                     flatten => 1,
-                  }
-               );
+
+   # so that, you can pass parameters, apply filters etc.
+   my %inc = (INCLUDE => map { trim $_ } split RE_PIPE_SPLIT, $file );
+
+   if ( $self->io->file_exists( $inc{INCLUDE} ) ) {
+      # well... constantly working around :p
+      $inc{INCLUDE} = qq{'$inc{INCLUDE}'};
+   }
+
+   # die "You can not pass parameters to static includes"
+   #    if $inc{PARAM} && T_STATIC  == $type;
+
+   my $filter = $inc{FILTER} ? escape( q{'} => $inc{FILTER} ) : '';
+
+   my $rv = $self->_mini_compiler(
+               $self->_internal('sub_include') => {
+                  OBJECT      => $self->[FAKER_SELF],
+                  INCLUDE     => escape( q{'} => $inc{INCLUDE} ),
+                  ERROR_TITLE => escape( q{'} => $etitle ),
+                  TYPE        => $type,
+                  PARAMS      => $inc{PARAM} ? qq{[$inc{PARAM}]} : 'undef',
+                  FILTER      => $filter,
+               } => {
+                  flatten => 1,
+               }
+            );
    return $rv;
 }
 
 sub _include_error {
    my $self  = shift;
    my $type  = shift;
-   my $title = '[ ' . $type . ' include error ]';
+   my $val   = T_DYNAMIC == $type ? 'dynamic'
+             : T_STATIC  == $type ? 'static'
+             :                      'unknown'
+             ;
+   my $title = '[ ' . $val . ' include error ]';
    return $title;
 }
 
@@ -151,6 +175,9 @@ Private module.
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Base::Include>
+released on C<26 April 2009>.
+
 Private module.
 
 =head1 AUTHOR
