@@ -1,11 +1,11 @@
 package Text::Template::Simple::Base::Parser;
 use strict;
 use vars qw($VERSION);
-use Carp qw( croak );
-use Text::Template::Simple::Util;
-use Text::Template::Simple::Constants;
 
-$VERSION = '0.60';
+$VERSION = '0.70';
+
+use Text::Template::Simple::Util qw(:all);
+use Text::Template::Simple::Constants qw(:all);
 
 # internal code templates
 my %INTERNAL = (
@@ -28,9 +28,10 @@ my %INTERNAL = (
             }
             $rv;
          },
-         undef,
+         <%PARAMS%>,
          {
-            _sub_inc => '<%TYPE%>'
+            _sub_inc => '<%TYPE%>',
+            _filter  => '<%FILTER%>',
          }
       )
    ~,
@@ -63,12 +64,55 @@ my %INTERNAL = (
    map_keys_default => q(
       <%BUF%> .= <%HASH%>->{"<%KEY%>"};
    ),
+
+   add_sigwarn => q(
+      my <%BUF%>;
+      local $SIG{__WARN__} = sub {
+         push @{ <%BUF%> }, $_[0];
+      };
+   ),
+   dump_sigwarn => q(
+      join("\n",
+            map {
+               s{ \A \s+    }{}xms;
+               s{    \s+ \z }{}xms;
+               "[warning] $_\n"
+            } @{ <%BUF%> }
+         );
+   ),
+
+   compile_error => <<'TEMPLATE_CONSTANT',
+Error compiling code fragment (cache id: <%CID%>):
+
+<%ERROR%>
+-------------------------------
+PARSED CODE (VERBATIM):
+-------------------------------
+
+<%PARSED%>
+
+-------------------------------
+PARSED CODE    (tidied):
+-------------------------------
+
+<%TIDIED%>
+TEMPLATE_CONSTANT
+
+   fragment => <<'TEMPLATE_CONSTANT',
+
+# BEGIN TIDIED FRAGMENT
+
+<%FRAGMENT%>
+
+# END TIDIED FRAGMENT
+TEMPLATE_CONSTANT
+
 );
 
 sub _internal {
    my $self = shift;
-   my $id   = shift            || croak "_internal(): id is missing";
-   my $rv   = $INTERNAL{ $id } || croak "_internal(): id is invalid";
+   my $id   = shift            || fatal('tts.base.parser._internal.id');
+   my $rv   = $INTERNAL{ $id } || fatal('tts.base.parser._internal.id');
    return $rv;
 }
 
@@ -97,10 +141,13 @@ sub _parse {
 
    LOG( RAW => $raw ) if ( DEBUG() > 3 );
 
-   my $handler = $self->[USER_THANDLER];
+   my $uthandler = $self->[USER_THANDLER];
 
-   my $w_raw = sub { ";$faker .= q~$_[0]~;" };
-   my $w_cap = sub { ";$faker .= sub {" . $_[0] . "}->();"; };
+   my $h = {
+      raw     => sub { ";$faker .= q~$_[0]~;" },
+      capture => sub { ";$faker .= sub {" . $_[0] . "}->();"; },
+      code    => sub { $_[0] . ';' },
+   };
 
    # little hack to convert delims into escaped delims for static inclusion
    $raw =~ s{\Q$ds}{$ds!}xmsg if $as_is;
@@ -108,47 +155,63 @@ sub _parse {
    # fetch and walk the tree
    PARSER: foreach my $token ( @{ $toke->tokenize( $raw, $map_keys ) } ) {
       my($str, $id, $chomp, undef) = @{ $token };
-      LOG( TOKEN => "$id => $str" ) if DEBUG() > 1;
-      next PARSER if $id eq 'DISCARD';
-      next PARSER if $id eq 'COMMENT';
+      LOG( TOKEN => $toke->_visualize_tid($id) . " => $str" ) if DEBUG() > 1;
+      next PARSER if T_DISCARD == $id;
+      next PARSER if T_COMMENT == $id;
 
-      if ( $id eq 'DELIMSTART' ) { $inside++; next PARSER; }
-      if ( $id eq 'DELIMEND'   ) { $inside--; next PARSER; }
+      if ( T_DELIMSTART == $id ) { $inside++; next PARSER; }
+      if ( T_DELIMEND   == $id ) { $inside--; next PARSER; }
 
-      if ( $id eq 'RAW' || $id eq 'NOTADELIM' ) {
-         $code .= $w_raw->( $self->_chomp( $str, $chomp ) );
+      if ( T_RAW == $id || T_NOTADELIM == $id ) {
+         $code .= $h->{raw}->( $self->_chomp( $str, $chomp ) );
       }
 
-      elsif ( $id eq 'CODE' ) {
-         $code .= $resume ? $self->_resume($str, 0, 1) : $str;
+      elsif ( T_CODE == $id ) {
+         $code .= $h->{code}->($resume ? $self->_resume($str, 0, 1) : $str);
       }
 
-      elsif ( $id eq 'CAPTURE' ) {
+      elsif ( T_CAPTURE == $id ) {
          $code .= $faker;
          $code .= $resume ? $self->_resume($str, RESUME_NOSTART)
                 :           " .= sub { $str }->();";
       }
 
-      elsif ( $id eq 'DYNAMIC' || $id eq 'STATIC' ) {
+      elsif ( T_DYNAMIC == $id || T_STATIC == $id ) {
          $self->[NEEDS_OBJECT]++;
-         $code .= $w_cap->( $self->_include($id, $str) );
+         $code .= $h->{capture}->( $self->_include($id, $str) );
       }
 
-      elsif ( $id eq 'MAPKEY' ) {
+      elsif ( T_MAPKEY == $id ) {
          $code .= sprintf $mko, $mkc ? ( ($str) x 5 ) : $str;
       }
 
+      elsif ( T_COMMAND == $id ) {
+         my($head, $raw_block) = split /;/, $str, 2;
+         my @buf = split RE_PIPE_SPLIT, '|' . trim($head);
+         shift(@buf);
+         my %com = map { trim $_ } @buf;
+
+         if ( $com{FILTER} ) {
+            # embed into the template & NEEDS_OBJECT++ ???
+            local $self->[FILENAME] = '<ANON BLOCK>';
+            $self->_call_filters(
+               \$raw_block,
+               split RE_FILTER_SPLIT, $com{FILTER}
+            );
+         }
+
+         $code .= $h->{raw}->($raw_block);
+      }
+
       else {
-         if ( $handler ) {
-            LOG( USER_THANDLER => "$id") if DEBUG;
-            $code .= $handler->(
-                        $self, $id ,$str, { capture => $w_cap, raw => $w_raw }
-                     );
+         if ( $uthandler ) {
+            LOG( USER_THANDLER => "$id") if DEBUG();
+            $code .= $uthandler->( $self, $id ,$str, $h );
          }
          else {
             LOG( UNKNOWN_TOKEN => "Adding unknown token as RAW: $id($str)")
-               if DEBUG;
-            $code .= $w_raw->($str);
+               if DEBUG();
+            $code .= $h->{raw}->($str);
          }
       }
 
@@ -156,13 +219,14 @@ sub _parse {
 
    $self->[FILENAME] ||= '<ANON>';
 
-   if ( $inside ) {
-      my $type = $inside > 0 ? 'opening' : 'closing';
-      my $tmpl = "%d unbalanced %s delimiter(s) in template %s";
-      croak sprintf( $tmpl, abs($inside), $type, $self->[FILENAME] );
-   }
+   fatal(
+      'tts.base.parser._parse.unbalanced',
+         abs($inside),
+         ($inside > 0 ? 'opening' : 'closing'),
+         $self->[FILENAME]
+   ) if $inside;
 
-   return $self->_wrapper( $code, $cache_id, $faker, $map_keys );
+   return $self->_wrapper( $code, $cache_id, $faker, $map_keys, $h );
 }
 
 sub _chomp {
@@ -202,6 +266,7 @@ sub _wrapper {
    my $cache_id = shift;
    my $faker    = shift;
    my $map_keys = shift;
+   my $h        = shift;
    my $buf_hash = $self->[FAKER_HASH];
 
    my $wrapper    = '';
@@ -223,14 +288,20 @@ sub _wrapper {
    $wrapper .= "my $faker = '';";
    $wrapper .= $self->_add_stack( $cache_id )  if $self->[STACK];
    $wrapper .= "my $buf_hash = {\@_};"         if $map_keys;
+   $wrapper .= $self->_add_sigwarn if $self->[CAPTURE_WARNINGS];
    $wrapper .= "\n#line 1 " .  $self->[FILENAME] . "\n";
-   $wrapper .= $code . ";return $faker;";
+   $wrapper .= $code . ";";
+   $wrapper .= $self->_dump_sigwarn($h) if $self->[CAPTURE_WARNINGS];
+   $wrapper .= "return $faker;";
    $wrapper .= '}';
    # make this a capture sub if we're including
    $wrapper .= '->()' if $inside_inc;
 
-   LOG( COMPILED => sprintf FRAGMENT_TMP, $self->_tidy($wrapper) )
-      if DEBUG() > 1;
+   LOG( COMPILED =>  $self->_mini_compiler(
+                        $self->_internal('fragment'),
+                        { FRAGMENT => $self->_tidy($wrapper) }
+                     )
+   ) if DEBUG > 1;
    #LOG( OUTPUT => $wrapper );
    # reset
    $self->[DEEP_RECURSION] = 0 if $self->[DEEP_RECURSION];
@@ -259,6 +330,30 @@ sub _parse_mapkeys {
    return $mko, $mkc;
 }
 
+sub _add_sigwarn {
+   my $self = shift;
+   $self->[FAKER_WARN] = $self->_output_buffer_var('array');
+   my $rv = $self->_mini_compiler(
+               $self->_internal('add_sigwarn'),
+               { BUF     => $self->[FAKER_WARN] },
+               { flatten => 1                   }
+            );
+   return $rv;
+}
+
+sub _dump_sigwarn {
+   my $self = shift;
+   my $h    = shift;
+   my $rv = $h->{capture}->(
+               $self->_mini_compiler(
+                  $self->_internal('dump_sigwarn'),
+                  { BUF     => $self->[FAKER_WARN] },
+                  { flatten => 1                   }
+               )
+            );
+   return $rv;
+}
+
 sub _add_stack {
    my $self    = shift;
    my $cs_name = shift || '<ANON TEMPLATE>';
@@ -335,6 +430,9 @@ Private module.
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Base::Parser>
+released on C<26 April 2009>.
+
 Private module.
 
 =begin CHOMPING
@@ -356,7 +454,8 @@ If the previous or next is not raw, nothing will happen. You need to swap sides
 when handling the chomping. i.e.: left chomping affects the right side of the
 RAW, and right chomping affects the left side of the RAW. _chomp() method in
 the parser swaps sides to handle chomping.
-See Text::Template::Simple::Tokenizer to see how pre-parsing happens.
+See Text::Template::Simple::Tokenizer to have an idea on how pre-parsing
+happens.
 
 =end CHOMPING
 
