@@ -2,17 +2,17 @@ package Text::Template::Simple::Cache;
 use strict;
 use vars qw($VERSION);
 use constant CACHE_PARENT => 0;
-use Text::Template::Simple::Constants;
-use Text::Template::Simple::Util qw( DEBUG LOG ishref );
+use Text::Template::Simple::Constants qw(:all);
+use Text::Template::Simple::Util qw( DEBUG LOG ishref fatal );
 use Carp qw( croak );
 
-$VERSION = '0.53';
+$VERSION = '0.70';
 
 my $CACHE = {}; # in-memory template cache
 
 sub new {
    my $class  = shift;
-   my $parent = shift || croak "Parent object is missing";
+   my $parent = shift || fatal('tts.cache.new.parent');
    my $self   = [undef];
    bless $self, $class;
    $self->[CACHE_PARENT] = $parent;
@@ -42,7 +42,7 @@ sub reset {
 
       my $cdir = $parent->[CACHE_DIR];
       local  *CDIRH;
-      opendir CDIRH, $cdir or croak fatal( CDIROPEN => $cdir, $! );
+      opendir CDIRH, $cdir or fatal( 'tts.cache.opendir' => $cdir, $! );
       require File::Spec;
       my $ext = quotemeta CACHE_EXT;
       my $file;
@@ -62,9 +62,9 @@ sub dumper {
    my $self  = shift;
    my $type  = shift || 'structure';
    my $param = shift || {};
-   croak "Parameters to dumper() must be a HASHref" if not ishref $param;
+   fatal('tts.cache.dumper.hash')        if not ishref $param;
    my %valid = map { $_, $_ } qw( ids structure );
-   croak "dumper type '$type' is not valid " if not $valid{ $type };
+   fatal('tts.cache.dumper.type', $type) if not $valid{ $type };
    my $method = '_dump_' . $type;
    return $self->$method( $param ); # TODO: modify the methods to accept HASH
 }
@@ -119,7 +119,7 @@ sub _dump_structure {
    else {
       $d = Data::Dumper->new( [ $CACHE ], [ $VAR ]);
       if ( $deparse ) {
-         croak fatal(DUMPER => $Data::Dumper::VERSION) if !$d->can('Deparse');
+         fatal('tts.cache.dumper' => $Data::Dumper::VERSION) if !$d->can('Deparse');
          $d->Deparse(1);
       }
    }
@@ -201,14 +201,16 @@ sub size {
       return $total;
 
    }
-   else { # in memory cache
+   else { # in-memory cache
 
       local $SIG{__DIE__};
       if ( eval { require Devel::Size; 1; } ) {
-         LOG( DEBUG => "Devel::Size v$Devel::Size::VERSION is loaded." )
+         my $dsv = Devel::Size->VERSION;
+         LOG( DEBUG => "Devel::Size v$dsv is loaded." )
             if DEBUG();
+         fatal('tts.cache.develsize.buggy', $dsv) if $dsv < 0.72;
          my $size = eval { Devel::Size::total_size( $CACHE ) };
-         die "Devel::Size::total_size(): $@" if $@;
+         fatal('tts.cache.develsize.total', $@) if $@;
          return $size;
       }
       else {
@@ -228,13 +230,13 @@ sub has {
       return;
    }
 
-   croak fatal('PFORMAT') if @_ % 2;
+   fatal('tts.cache.pformat') if @_ % 2;
 
    my %opt = @_;
    my $id  = $parent->connector('Cache::ID')->new;
    my $cid = $opt{id}   ? $id->generate($opt{id}  , 'custom')
            : $opt{data} ? $id->generate($opt{data}          )
-           :              croak fatal('INCACHE');
+           :              fatal('tts.cache.incache');
 
    if ( my $cdir = $parent->[CACHE_DIR] ) {
       require File::Spec;
@@ -246,6 +248,7 @@ sub has {
 }
 
 sub hit {
+   # TODO: return $CODE, $META;
    my $self     = shift;
    my $parent   = $self->[CACHE_PARENT];
    my $cache_id = shift;
@@ -258,20 +261,24 @@ sub hit {
 
       if ( -e $cache && not -d _ && -f _ ) {
          my $disk_cache = $parent->io->slurp($cache);
-
-         if ( $chkmt ) {
-            if ( $disk_cache =~ m{^#(\d+)#} ) {
-               my $mtime  = $1;
-               if ( $mtime != $chkmt ) {
-                  LOG( MTIME_DIFF => "\tOLD: $mtime\n\t\tNEW: $chkmt")
-                     if DEBUG();
-                  return; # i.e.: Update cache
-               }
+         my %meta;
+         if ( $disk_cache =~ m{ \A \#META: (.+?) \n }xms ) {
+            %meta = $self->_get_meta( $1 );
+            fatal('tts.cache.hit.meta', $@) if $@;
+         }
+         if ( my $mtime = $meta{CHKMT} ) {
+            if ( $mtime != $chkmt ) {
+               LOG( MTIME_DIFF => "\tOLD: $mtime\n\t\tNEW: $chkmt")
+                  if DEBUG();
+               return; # i.e.: Update cache
             }
          }
 
          ($CODE, $error) = $parent->_wrap_compile($disk_cache);
-         croak "Error loading from disk cache: $error" if $error;
+         $parent->[NEEDS_OBJECT] = $meta{NEEDS_OBJECT} if $meta{NEEDS_OBJECT};
+         $parent->[FAKER_SELF]   = $meta{FAKER_SELF}   if $meta{FAKER_SELF};
+
+         fatal('tts.cache.hit.cache', $error) if $error;
          LOG( FILE_CACHE => '' ) if DEBUG();
          #$parent->[COUNTER]++;
          return $CODE;
@@ -308,14 +315,20 @@ sub populate {
          require Fcntl;
          require IO::File;
 
+         my %meta = (
+            CHKMT        => $chkmt,
+            NEEDS_OBJECT => $parent->[NEEDS_OBJECT],
+            FAKER_SELF   => $parent->[FAKER_SELF],
+         );
+
          my $cache = File::Spec->catfile( $cdir, $cache_id . CACHE_EXT);
          my $fh    = IO::File->new;
-         $fh->open($cache, '>') or croak "Error writing disk-cache $cache : $!";
+         $fh->open($cache, '>') or fatal('tts.cache.populate.write', $cache, $!);
          flock $fh, Fcntl::LOCK_EX() if IS_FLOCK;
          $parent->io->layer($fh);
-         print $fh $chkmt ? "#$chkmt#\n" : "##\n",
+         print $fh '#META:' . $self->_set_meta(\%meta) . "\n",
                    sprintf( DISK_CACHE_COMMENT,
-                            PARENT->_parser_id, scalar localtime time),
+                            PARENT->_class_id, scalar localtime time),
                    $parsed; 
          flock $fh, Fcntl::LOCK_UN() if IS_FLOCK;
          close $fh;
@@ -324,10 +337,12 @@ sub populate {
          LOG( DISK_POPUL => $cache_id ) if DEBUG() > 2;
       } 
       else {
-         $CACHE->{$cache_id} = {CODE => undef, MTIME => 0}; # init
-         ($CODE, $error) = $parent->_wrap_compile($parsed);
-         $CACHE->{$cache_id}->{CODE}  = $CODE;
-         $CACHE->{$cache_id}->{MTIME} = $chkmt if $chkmt;
+         $CACHE->{ $cache_id } = {}; # init
+         ($CODE, $error)                       = $parent->_wrap_compile($parsed);
+         $CACHE->{ $cache_id }->{CODE}         = $CODE;
+         $CACHE->{ $cache_id }->{MTIME}        = $chkmt if $chkmt;
+         $CACHE->{ $cache_id }->{NEEDS_OBJECT} = $parent->[NEEDS_OBJECT];
+         $CACHE->{ $cache_id }->{FAKER_SELF}   = $parent->[FAKER_SELF];
          LOG( MEM_POPUL => $cache_id ) if DEBUG() > 2;
       }
    }
@@ -339,16 +354,39 @@ sub populate {
    if ( $error ) {
       my $cid    = $cache_id ? $cache_id : 'N/A';
       my $tidied = $parent->_tidy( $parsed );
-      croak sprintf COMPILE_ERROR_TMP, $cid, $error, $parsed, $tidied;
+      croak $parent->[VERBOSE_ERRORS]
+            ? $parent->_mini_compiler( $parent->_internal('compile_error'), {
+               CID => $cid,
+               ERROR => $error,
+               PARSED => $parsed,
+               TIDIED => $tidied,
+                                                                        } )
+            : $error
+            ;
    }
 
    $parent->[COUNTER]++;
    return $CODE;
 }
 
+sub _get_meta {
+   my $self = shift;
+   my $raw  = shift;
+   my %meta = map { split /:/, $_ } split /\|/, $raw;
+   return %meta;
+}
+
+sub _set_meta {
+   my $self = shift;
+   my $meta = shift;
+   my $rv   = join '|', map { $_ . ':' . $meta->{ $_ } } keys %{ $meta };
+   return $rv;
+}
+
 sub DESTROY {
    my $self = shift;
-   LOG( DESTROY => ref $self ) if DEBUG;
+   LOG( DESTROY => ref $self ) if DEBUG();
+   $self->[CACHE_PARENT] = undef;
    @{$self} = ();
    return;
 }
@@ -367,6 +405,9 @@ TODO
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Cache>
+released on C<26 April 2009>.
+
 Cache manager for C<Text::Template::Simple>.
 
 =head1 METHODS
