@@ -11,15 +11,14 @@ use constant EVALTEXT   => 6;
 use constant IS_REQUIRE => 7;
 use constant HINTS      => 8;
 use constant BITMASK    => 9;
+use Text::Template::Simple::Util qw( ishref fatal );
 
-use Carp qw( croak );
-
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 sub stack {
    my $self    = shift;
    my $opt     = shift || {};
-   die "Parameters to stack() must be a HASH" if ref($opt) ne 'HASH';
+   fatal('tts.caller.stack.hash') if ! ishref($opt);
    my $frame   = $opt->{frame} || 0;
    my $type    = $opt->{type}  || '';
    my(@callers, $context);
@@ -58,7 +57,7 @@ sub stack {
       return $self->$method( $opt, \@callers );
    }
 
-   croak "Unknown caller stack type: $type";
+   fatal('tts.caller.stack.type', $type);
 }
 
 sub _string {
@@ -141,7 +140,7 @@ sub _text_table {
    my $opt     = shift;
    my $callers = shift;
    eval { require Text::Table; };
-   croak "Caller stack type 'text_table' requires Text::Table" if $@;
+   fatal('tts.caller._text_table.module', $@) if $@;
 
    my $table = Text::Table->new( qw(
                   | CONTEXT    | SUB      | LINE  | FILE    | HASARGS
@@ -198,6 +197,9 @@ Text::Template::Simple::Caller - Caller stack tracer
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Caller>
+released on C<26 April 2009>.
+
 Caller stack tracer for Text::Template::Simple. This module is not used
 directly inside templates. You must use the global template function
 instead. See L<Text::Template::Simple::Dummy> for usage from the templates.
