@@ -1,8 +1,8 @@
 package Text::Template::Simple::Constants;
 use strict;
-use vars qw($VERSION $OID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
+use vars qw($VERSION $OID $DID @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 # object fields
 BEGIN { $OID = -1 } # init object field id counter
@@ -12,6 +12,7 @@ use constant DELETE_WS        => ++$OID;
 use constant FAKER            => ++$OID;
 use constant FAKER_HASH       => ++$OID;
 use constant FAKER_SELF       => ++$OID;
+use constant FAKER_WARN       => ++$OID;
 use constant MONOLITH         => ++$OID;
 use constant CACHE            => ++$OID;
 use constant CACHE_DIR        => ++$OID;
@@ -21,6 +22,7 @@ use constant STRICT           => ++$OID;
 use constant SAFE             => ++$OID;
 use constant HEADER           => ++$OID;
 use constant ADD_ARGS         => ++$OID;
+use constant CAPTURE_WARNINGS => ++$OID;
 use constant WARN_IDS         => ++$OID;
 use constant TYPE             => ++$OID;
 use constant TYPE_FILE        => ++$OID;
@@ -38,8 +40,25 @@ use constant DEEP_RECURSION   => ++$OID;
 use constant INCLUDE_PATHS    => ++$OID;
 use constant PRE_CHOMP        => ++$OID;
 use constant POST_CHOMP       => ++$OID;
+use constant VERBOSE_ERRORS   => ++$OID;
 use constant MAXOBJFIELD      =>   $OID; # number of the last object field
 
+# token type ids
+BEGIN { $DID = 0 }
+use constant T_DELIMSTART     => ++$DID;
+use constant T_DELIMEND       => ++$DID;
+use constant T_DISCARD        => ++$DID;
+use constant T_COMMENT        => ++$DID;
+use constant T_RAW            => ++$DID;
+use constant T_NOTADELIM      => ++$DID;
+use constant T_CODE           => ++$DID;
+use constant T_CAPTURE        => ++$DID;
+use constant T_DYNAMIC        => ++$DID;
+use constant T_STATIC         => ++$DID;
+use constant T_MAPKEY         => ++$DID;
+use constant T_COMMAND        => ++$DID;
+use constant T_MAXID          =>   $DID;
+
 # settings
 use constant MAX_RECURSION    => 50; # recursion limit for dynamic includes
 use constant PARENT           => 'Text::Template::Simple';
@@ -75,6 +94,7 @@ use constant DIR_DYNAMIC      => '*';
 use constant DIR_STATIC       => '+';
 use constant DIR_NOTADELIM    => '!';
 use constant DIR_COMMENT      => '#';
+use constant DIR_COMMAND      => '|';
 # second level directives
 use constant DIR_CHOMP        => '-';
 use constant DIR_COLLAPSE     => '~';
@@ -106,31 +126,8 @@ use constant DIGEST_MODS => qw(
    Digest::Perl::MD5
 );
 
-use constant FRAGMENT_TMP      => <<'TEMPLATE_CONSTANT';
-
-# BEGIN TIDIED FRAGMENT
-
-%s
-
-# END TIDIED FRAGMENT
-TEMPLATE_CONSTANT
-
-use constant COMPILE_ERROR_TMP => <<'TEMPLATE_CONSTANT';
-Error compiling code fragment (cache id: %s):
-
-%s
--------------------------------
-PARSED CODE (VERBATIM):
--------------------------------
-
-%s
-
--------------------------------
-PARSED CODE    (tidied):
--------------------------------
-
-%s
-TEMPLATE_CONSTANT
+use constant RE_PIPE_SPLIT   => qr/ \| (?:\s+)? (PARAM|FILTER) : /xms;
+use constant RE_FILTER_SPLIT => qr/ \, (?:\s+)? /xms;
 
 use constant DISK_CACHE_COMMENT => <<"TEMPLATE_CONSTANT";
 # !!!   W A R N I N G      W A R N I N G      W A R N I N G   !!!
@@ -193,10 +190,7 @@ use constant RESUME_ELSE => qr{
 
 use constant RESUME_LOOP => qr{ (?:next|last|continue|redo) }xms;
 
-use Exporter ();
-
 BEGIN {
-
    @ISA         = qw( Exporter );
 
    %EXPORT_TAGS = (
@@ -212,8 +206,6 @@ BEGIN {
                         PARENT
                      )],
       templates =>   [qw(
-                        COMPILE_ERROR_TMP
-                        FRAGMENT_TMP
                         DISK_CACHE_COMMENT
                         DISK_CACHE_MARKER
                      )],
@@ -229,6 +221,7 @@ BEGIN {
                         FAKER
                         FAKER_HASH
                         FAKER_SELF
+                        FAKER_WARN
                         CACHE
                         CACHE_DIR
                         CACHE_OBJECT
@@ -239,6 +232,7 @@ BEGIN {
                         HEADER
                         ADD_ARGS
                         WARN_IDS
+                        CAPTURE_WARNINGS
                         TYPE
                         TYPE_FILE
                         COUNTER
@@ -255,6 +249,7 @@ BEGIN {
                         INCLUDE_PATHS
                         PRE_CHOMP
                         POST_CHOMP
+                        VERBOSE_ERRORS
                         MAXOBJFIELD
                      )],
       resume    =>   [qw(
@@ -285,6 +280,7 @@ BEGIN {
                         DIR_STATIC
                         DIR_NOTADELIM
                         DIR_COMMENT
+                        DIR_COMMAND
                      )],
       token     =>   [qw(
                         TOKEN_ID
@@ -295,13 +291,29 @@ BEGIN {
                         TOKEN_CHOMP_PREV
                         LAST_TOKEN
                         PREVIOUS_TOKEN
+
+                        T_DELIMSTART
+                        T_DELIMEND
+                        T_DISCARD
+                        T_COMMENT
+                        T_RAW
+                        T_NOTADELIM
+                        T_CODE
+                        T_CAPTURE
+                        T_DYNAMIC
+                        T_STATIC
+                        T_MAPKEY
+                        T_COMMAND
+                        T_MAXID
                       )],
       etc       =>   [qw(
                         DIGEST_MODS
                         STAT_MTIME
                         RE_DUMP_ERROR
-                        STAT_SIZE
+                        RE_PIPE_SPLIT
+                        RE_FILTER_SPLIT
                         RE_NONFILE
+                        STAT_SIZE
                         MAX_RECURSION
                      )],
    );
@@ -312,6 +324,8 @@ BEGIN {
 
 }
 
+BEGIN { require Exporter; }
+
 1;
 
 __END__
@@ -326,6 +340,9 @@ TODO
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Constants>
+released on C<26 April 2009>.
+
 Constants for Text::Template::Simple.
 
 =head1 AUTHOR
