@@ -8,12 +8,14 @@ package Text::Template::Simple::Dummy;
 use strict;
 use vars qw($VERSION);
 use Text::Template::Simple::Caller;
+use Text::Template::Simple::Util qw();
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 sub stack { # just a wrapper
    my $opt = shift || {};
-   die "Parameters to stack() must be a HASH" if ref($opt) ne 'HASH';
+   Text::Template::Simple::Util::fatal('tts.caller.stack.hash')
+      if ! Text::Template::Simple::Util::ishref($opt);
    $opt->{frame} = 1;
    Text::Template::Simple::Caller->stack( $opt );
 }
@@ -32,6 +34,9 @@ TODO
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Dummy>
+released on C<26 April 2009>.
+
 All templates are compiled into this class.
 
 =head1 FUNCTIONS
