@@ -1,24 +1,29 @@
 package Text::Template::Simple::IO;
 use strict;
 use vars qw($VERSION);
-use Text::Template::Simple::Constants;
-use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode );
-use Carp qw( croak );
+use File::Spec;
+use Text::Template::Simple::Constants qw(:all);
+use Text::Template::Simple::Util qw( DEBUG LOG ishref binary_mode fatal );
+use constant MY_IO_LAYER      => 0;
+use constant MY_INCLUDE_PATHS => 1;
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 sub new {
    my $class = shift;
    my $layer = shift;
-   my $self  = bless do { \my $anon }, $class;
-   $$self    = $layer if defined $layer;
+   my $paths = shift;
+   my $self  = [ undef, undef ];
+   bless $self, $class;
+   $self->[MY_IO_LAYER]      = $layer if defined $layer;
+   $self->[MY_INCLUDE_PATHS] = [ @{ $paths } ] if $paths; # copy
    $self;
 }
 
 sub validate {
    my $self = shift;
-   my $type = shift || croak "No type specified";
-   my $path = shift || croak "No path specified";
+   my $type = shift || fatal('tts.io.validate.type');
+   my $path = shift || fatal('tts.io.validate.path');
 
    if ( $type eq 'dir' ) {
       require File::Spec;
@@ -29,7 +34,7 @@ sub validate {
          $wdir = Win32::GetFullPathName( $path );
          if( Win32::GetLastError() ) {
             LOG( FAIL => "Win32::GetFullPathName( $path ): $^E" ) if DEBUG();
-            $wdir = ''; # croak "Win32::GetFullPathName: $^E";
+            $wdir = ''; # die "Win32::GetFullPathName: $^E";
          }
          else {
             my $ok = -e $wdir && -d _;
@@ -43,14 +48,14 @@ sub validate {
       return $path;
    }
 
-   croak "validate(file) is not yet implemented";
+   fatal('tts.io.validate.file');
 }
 
 sub layer {
    return if ! NEW_PERL;
    my $self   = shift;
-   my $fh     = shift || croak "layer(): Filehandle is absent";
-   my $layer  = $$self; # || croak "_iolayer(): I/O Layer is absent";
+   my $fh     = shift || fatal('tts.io.layer.fh');
+   my $layer  = $self->[MY_IO_LAYER];
    binary_mode( $fh, $layer ) if $layer;
    return;
 }
@@ -61,7 +66,7 @@ sub slurp {
    my $self = shift;
    my $file = shift;
    my($fh, $seek);
-   LOG(IO_SLURP => $file) if DEBUG;
+   LOG(IO_SLURP => $file) if DEBUG();
 
    if ( fileno $file ) {
       $fh   = $file;
@@ -69,7 +74,7 @@ sub slurp {
    }
    else {
       $fh = IO::File->new;
-      $fh->open($file, 'r') or croak "Error opening '$file' for reading: $!";
+      $fh->open($file, 'r') or fatal('tts.io.slurp.open', $file, $!);
    }
 
    flock $fh,    Fcntl::LOCK_SH()  if IS_FLOCK;
@@ -83,21 +88,46 @@ sub slurp {
 
 sub is_file {
    # safer than a simple "-e"
+   my $self = shift;
+   my $file = shift || return;
+   return $self->_looks_like_file( $file ) && ! -d $file;
+}
+
+sub is_dir {
+   # safer than a simple "-d"
+   my $self = shift;
+   my $file = shift || return;
+   return $self->_looks_like_file( $file ) && -d $file;
+}
+
+sub file_exists {
+   my $self = shift;
+   my $file = shift;
+
+   return $file if $self->is_file( $file );
+
+   foreach my $path ( @{ $self->[MY_INCLUDE_PATHS] } ) {
+      my $test = File::Spec->catfile( $path, $file );
+      return $test if $self->is_file( $test );
+   }
+
+   return; # fail!
+}
+
+sub _looks_like_file {
    my $self = shift;
    my $file = shift || return;
    return     ref $file               ? 0
          :        $file =~ RE_NONFILE ? 0
          : length $file >= 255        ? 0
-         : ! -e   $file               ? 0
-         :   -d _                     ? 0
-         :                              1
+         :     -e $file               ? 1
+         :                              0
          ;
 }
 
 sub DESTROY {
    my $self = shift;
-   LOG( DESTROY => ref $self ) if DEBUG;
-   $$self = undef;
+   LOG( DESTROY => ref $self ) if DEBUG();
    return;
 }
 
@@ -115,6 +145,9 @@ TODO
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::IO>
+released on C<26 April 2009>.
+
 TODO
 
 =head1 METHODS
@@ -137,10 +170,19 @@ Returns the contents of the supplied file as a string.
 C<TYPE> can either be C<dir> or C<file>. Returns the corrected path if
 it is valid, C<undef> otherwise.
 
+=head2 is_dir THING
+
+Test if C<THING> is a directory.
+
 =head2 is_file THING
 
 Test if C<THING> is a file.
 
+=head2 file_exists THING
+
+Test if C<THING> is a file. This method also searches all the C<include paths>
+and returns the full path to the file if it exists.
+
 =head1 AUTHOR
 
 Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
