@@ -2,40 +2,37 @@ package Text::Template::Simple::Tokenizer;
 use strict;
 use vars qw($VERSION);
 
-$VERSION = '0.60';
+$VERSION = '0.70';
 
 use constant CMD_CHAR             =>  0;
 use constant CMD_ID               =>  1;
-use constant CMD_CB               =>  2; # callback
-
+use constant CMD_CB               =>  2; # callbacks
 use constant ID_DS                =>  0;
 use constant ID_DE                =>  1;
 use constant ID_PRE_CHOMP         =>  2;
 use constant ID_POST_CHOMP        =>  3;
-
 use constant SUBSTR_OFFSET_FIRST  =>  0;
 use constant SUBSTR_OFFSET_SECOND =>  1;
 use constant SUBSTR_LENGTH        =>  1;
-
-use Carp qw( croak );
-use Text::Template::Simple::Util      qw( LOG );
+use Text::Template::Simple::Util      qw( LOG fatal );
 use Text::Template::Simple::Constants qw( :chomp :directive :token );
 
-my @COMMANDS = (
-   # cmd                      id        callback
-   [ DIR_CAPTURE  , qw/ CAPTURE         / ],
-   [ DIR_DYNAMIC  , qw/ DYNAMIC   trim  / ],
-   [ DIR_STATIC   , qw/ STATIC    trim  / ],
-   [ DIR_NOTADELIM, qw/ NOTADELIM       / ],
-   [ DIR_COMMENT  , qw/ COMMENT         / ],
+my @COMMANDS = ( # default command list
+   # cmd            id
+   [ DIR_CAPTURE  , T_CAPTURE   ],
+   [ DIR_DYNAMIC  , T_DYNAMIC,  ],
+   [ DIR_STATIC   , T_STATIC,   ],
+   [ DIR_NOTADELIM, T_NOTADELIM ],
+   [ DIR_COMMENT  , T_COMMENT   ],
+   [ DIR_COMMAND  , T_COMMAND   ],
 );
 
 sub new {
    my $class = shift;
    my $self  = [];
    bless $self, $class;
-   $self->[ID_DS]         = shift || croak "Start delimiter is missing";
-   $self->[ID_DE]         = shift || croak "End delimiter is missing";
+   $self->[ID_DS]         = shift || fatal('tts.tokenizer.new.ds');
+   $self->[ID_DE]         = shift || fatal('tts.tokenizer.new.de');
    $self->[ID_PRE_CHOMP]  = shift || CHOMP_NONE;
    $self->[ID_POST_CHOMP] = shift || CHOMP_NONE;
    $self;
@@ -44,17 +41,17 @@ sub new {
 sub tokenize {
    # compile the template into a tree and optimize
    my $self       = shift;
-   my $tmp        = shift || croak "tokenize(): Template string is missing";
+   my $tmp        = shift || fatal('tts.tokenizer.tokenize.tmp');
    my $map_keys   = shift;
    my($ds, $de)   = ($self->[ID_DS], $self->[ID_DE]);
    my($qds, $qde) = map { quotemeta $_ } $ds, $de;
 
-   my(@tokens, $inside, $last, $i, $j);
+   my(@tokens, $inside);
 
    OUT_TOKEN: foreach my $i ( split /($qds)/, $tmp ) {
 
       if ( $i eq $ds ) {
-         push @tokens, [ $i, 'DELIMSTART', [], undef ];
+         push @tokens, [ $i, T_DELIMSTART, [], undef ];
          $inside = 1;
          next OUT_TOKEN;
       }
@@ -62,11 +59,11 @@ sub tokenize {
       IN_TOKEN: foreach my $j ( split /($qde)/, $i ) {
          if ( $j eq $de ) {
             my $last = $tokens[LAST_TOKEN];
-            if ( $last->[TOKEN_ID] eq 'NOTADELIM' ) {
+            if ( T_NOTADELIM == $last->[TOKEN_ID] ) {
                $last->[TOKEN_STR] = $self->tilde( $last->[TOKEN_STR] . $de );
             }
             else {
-               push @tokens, [ $j, 'DELIMEND', [], undef ];
+               push @tokens, [ $j, T_DELIMEND, [], undef ];
             }
             $inside = 0;
             next IN_TOKEN;
@@ -82,7 +79,6 @@ sub tokenize {
 
 sub tilde { shift; Text::Template::Simple::Util::escape( '~' => @_ ) }
 sub quote { shift; Text::Template::Simple::Util::escape( '"' => @_ ) }
-sub trim  { shift; Text::Template::Simple::Util::trim(          @_ ) }
 
 sub _debug_tokens {
    my $self   = shift;
@@ -115,7 +111,8 @@ DUMP
          scalar $self->_visualize_chomp( $t->[TOKEN_CHOMP][TOKEN_CHOMP_PREV] ),
          scalar $self->_visualize_chomp( $t->[TOKEN_TRIGGER]                 )
       );
-      $buf .= sprintf $tmp, $t->[TOKEN_ID], $s, @v;
+      @v = map { $_ eq 'undef' ? '' : $_ } @v;
+      $buf .= sprintf $tmp, $self->_visualize_tid( $t->[TOKEN_ID] ), $s, @v;
    }
    Text::Template::Simple::Util::LOG( DEBUG => $buf );
 }
@@ -126,75 +123,84 @@ sub _user_commands {
    return $self->commands;
 }
 
-sub _token_code {
-   my $self     = shift;
-   my $str      = shift;
-   my $inside   = shift;
-   my $map_keys = shift;
-   my $tree     = shift;
-   # $first is the left-cmd, $last is the right-cmd. $second is the extra
-   my $first    = substr $str, SUBSTR_OFFSET_FIRST , SUBSTR_LENGTH;
-   my $second   = substr $str, SUBSTR_OFFSET_SECOND, SUBSTR_LENGTH;
-   my $last     = substr $str, length($str) - 1    , SUBSTR_LENGTH;
-   my $len      = length($str);
+sub _token_for_command {
+   my($self, $tree, $map_keys, $str, $last, $second, $cmd, $inside) = @_;
+   my($copen, $cclose, $ctoken) = $self->_chomp_token( $second, $last );
+   my $len  = length($str);
+   my $cb   = $map_keys ? 'quote' : $cmd->[CMD_CB];
+   my $soff = $copen ? 2 : 1;
+   my $slen = $len - ($cclose ? $soff+1 : 1);
+   my $buf  = substr $str, $soff, $slen;
 
-   HANDLE_OUTSIDE: {
-      foreach my $cmd ( @COMMANDS, $self->_user_commands ) {
-         if ( $first eq $cmd->[CMD_CHAR] ) {
-            my($copen, $cclose, $ctoken) = $self->_chomp_token( $second, $last );
-            my $cb   = $map_keys ? 'quote' : $cmd->[CMD_CB];
-            my $soff = $copen ? 2 : 1;
-            my $slen = $len - ($cclose ? $soff+1 : 1);
-            my $buf  = substr $str, $soff, $slen;
-
-            if ( $cmd->[CMD_ID] eq 'NOTADELIM' && $inside ) {
-               $buf = $self->[ID_DS] . $buf;
-               $tree->[LAST_TOKEN][TOKEN_ID] = 'DISCARD';
-            }
-
-            my $needs_chomp = defined($ctoken);
-            $self->_chomp_prev($tree, $ctoken) if $needs_chomp;
-
-            my $id  = $map_keys ? 'RAW'              : $cmd->[CMD_ID];
-            my $val = $cb       ? $self->$cb( $buf ) : $buf;
-
-            return [
-                     $val,
-                     $id,
-                     [CHOMP_NONE, CHOMP_NONE],
-                     $needs_chomp ? $ctoken : undef # trigger
-                   ];
-         }
-      }
+   if ( T_NOTADELIM == $cmd->[CMD_ID] ) {
+      $buf = $self->[ID_DS] . $buf;
+      $tree->[LAST_TOKEN][TOKEN_ID] = T_DISCARD;
    }
 
-   if ( $inside ) {
-      my($copen, $cclose, $ctoken) = $self->_chomp_token( $first, $last );
-      my $soff = $copen ? 1 : 0;
-      my $slen = $len - ( $cclose ? $soff+1 : 0 );
+   my $needs_chomp = defined($ctoken);
+   $self->_chomp_prev($tree, $ctoken) if $needs_chomp;
 
-      my $needs_chomp = defined($ctoken);
-      $self->_chomp_prev($tree, $ctoken) if $needs_chomp;
+   my $id  = $map_keys ? T_RAW              : $cmd->[CMD_ID];
+   my $val = $cb       ? $self->$cb( $buf ) : $buf;
 
-      return   [
-                  substr($str, $soff, $slen),
-                  $map_keys ? 'MAPKEY' : 'CODE',
-                  [ CHOMP_NONE, CHOMP_NONE ],
-                  $needs_chomp ? $ctoken : undef # trigger
-               ];
-   }
-
-   my $trig = $tree->[PREVIOUS_TOKEN] ? $tree->[PREVIOUS_TOKEN][TOKEN_TRIGGER]
-            :                           undef
-            ;
    return [
-            $self->tilde( $str ),
-            'RAW',
-            [ $trig, CHOMP_NONE ],
-            undef
-         ];
+            $val,
+            $id,
+            [CHOMP_NONE, CHOMP_NONE],
+            $needs_chomp ? $ctoken : undef # trigger
+          ];
 }
 
+sub _token_for_code {
+   my($self, $tree, $map_keys, $str, $last, $first) = @_;
+   my($copen, $cclose, $ctoken) = $self->_chomp_token( $first, $last );
+   my $len  = length($str);
+   my $soff = $copen ? 1 : 0;
+   my $slen = $len - ( $cclose ? $soff+1 : 0 );
+
+   my $needs_chomp = defined($ctoken);
+   $self->_chomp_prev($tree, $ctoken) if $needs_chomp;
+
+   return   [
+               substr($str, $soff, $slen),
+               $map_keys ? T_MAPKEY : T_CODE,
+               [ CHOMP_NONE, CHOMP_NONE ],
+               $needs_chomp ? $ctoken : undef # trigger
+            ];
+}
+
+sub _get_command_chars {
+   my($self, $str) = @_;
+   my($first, $second, $last) = ('') x 3;
+   # $first is the left-cmd, $last is the right-cmd. $second is the extra
+   $first  = substr $str, SUBSTR_OFFSET_FIRST , SUBSTR_LENGTH if $str ne '';
+   $second = substr $str, SUBSTR_OFFSET_SECOND, SUBSTR_LENGTH if $str ne '';
+   $last   = substr $str, length($str) - 1    , SUBSTR_LENGTH if $str ne '';
+   return $first, $second, $last;
+}
+
+sub _token_code {
+   my($self, $str, $inside, $map_keys, $tree) = @_;
+   my($first, $second, $last) = $self->_get_command_chars( $str );
+
+   if ( $inside ) {
+      my @common = ($tree, $map_keys, $str, $last);
+      foreach my $cmd ( @COMMANDS, $self->_user_commands ) {
+         next if $first ne $cmd->[CMD_CHAR];
+         return $self->_token_for_command( @common, $second, $cmd, $inside );
+      }
+      return $self->_token_for_code( @common, $first );
+   }
+
+   my $prev = $tree->[PREVIOUS_TOKEN];
+
+   return [
+            $self->tilde( $str ),
+            T_RAW,
+            [ $prev ? $prev->[TOKEN_TRIGGER] : undef, CHOMP_NONE ],
+            undef # trigger
+         ];
+}
 
 sub _chomp_token {
    my($self, $open, $close) = @_;
@@ -228,7 +234,7 @@ sub _chomp_token {
 sub _chomp_prev {
    my($self, $tree, $ctoken) = @_;
    my $prev = $tree->[PREVIOUS_TOKEN] || return; # no previous if this is first
-   return if $prev->[TOKEN_ID] ne 'RAW'; # only RAWs can be chomped
+   return if T_RAW != $prev->[TOKEN_ID]; # only RAWs can be chomped
 
    my $tc_prev = $prev->[TOKEN_CHOMP][TOKEN_CHOMP_PREV];
    my $tc_next = $prev->[TOKEN_CHOMP][TOKEN_CHOMP_NEXT];
@@ -284,6 +290,29 @@ sub _visualize_chomp {
    return $which, join( "\n", @test );
 }
 
+sub _visualize_tid {
+   my $self = shift;
+   my $id   = shift;
+   my @ids  = ( undef,
+                qw(
+                     T_DELIMSTART
+                     T_DELIMEND
+                     T_DISCARD
+                     T_COMMENT
+                     T_RAW
+                     T_NOTADELIM
+                     T_CODE
+                     T_CAPTURE
+                     T_DYNAMIC
+                     T_STATIC
+                     T_MAPKEY
+                     T_COMMAND
+                  )
+               );
+   my $rv = $ids[$id] || ( defined $id ? $id : 'undef' );
+   return $rv;
+}
+
 1;
 
 __END__
@@ -307,6 +336,9 @@ Text::Template::Simple::Tokenizer - Tokenizer
 
 =head1 DESCRIPTION
 
+This document describes version C<0.70> of C<Text::Template::Simple::Tokenizer>
+released on C<26 April 2009>.
+
 Tokenizes the input with the defined delimiter pair.
 
 =head1 METHODS
@@ -331,14 +363,6 @@ Escapes the tilde character.
 
 Escapes double quotes.
 
-=head3 trim
-
-=head3 rtrim
-
-=head3 ltrim
-
-See L<Text::Template::Simple::Util>.
-
 =head1 AUTHOR
 
 Burak GE<252>rsoy, E<lt>burakE<64>cpan.orgE<gt>
